#version 330 core

#define FISHEYE_STRENGTH 0.15
#define CHROMATIC_ABBERATION_STRENGTH 1.0
#define COLOR_GRADE_STRENGTH 0.5
#define VIGNETTE_DISTANCE 0.8
#define VIGNETTE_STRENGTH 2
#define BRIGHTNESS_SATURATION_STRENGTH 15 // Makes bright colors saturated, and darker colors desaturated.
#define MAX_SATURATION 1.5

uniform sampler2D scene;
in vec2 texCoord;
out vec4 outColor;

// https://github.com/hughsk/glsl-luma/blob/master/index.glsl
float luma(vec3 color) {
  return dot(color, vec3(0.299, 0.587, 0.114));
}

float luma(vec4 color) {
  return dot(color.rgb, vec3(0.299, 0.587, 0.114));
}

// adapted from https://github.com/CesiumGS/cesium/blob/main/Source/Shaders/Builtin/Functions/saturation.glsl
vec4 saturate(vec4 color, float adjustment)
{
	vec3 rgb = color.rgb;
    // Algorithm from Chapter 16 of OpenGL Shading Language
    const vec3 W = vec3(0.2125, 0.7154, 0.0721);
    vec3 intensity = vec3(dot(rgb, W));
    return vec4(mix(intensity, rgb, adjustment), color.a);
}

// https://www.shadertoy.com/view/MlXGRf
float SCurve(float value, float amount, float correction) {

	float curve = 1.0; 

    if (value < 0.5)
    {

        curve = pow(value, amount) * pow(2.0, amount) * 0.5; 
    }
        
    else
    { 	
    	curve = 1.0 - pow(1.0 - value, amount) * pow(2.0, amount) * 0.5; 
    }

    return pow(curve, correction);
}

// adapted from https://www.shadertoy.com/view/4s2GRR
vec4 fishEye(out vec4 fragColor, in vec2 fragCoord)
{
	// lens distortion
	vec2 iResolution = vec2(1.0, 1.0);
	vec2 p = fragCoord.xy * iResolution;
	                                    
	float prop = iResolution.x / iResolution.y;
	vec2 m = vec2(0.5, 0.5 / prop);
	vec2 d = p - m;
	float r = sqrt(dot(d, d));

	float power = ( 2.0 * 3.141592 / (2.0 * sqrt(dot(m, m))) ) * FISHEYE_STRENGTH;

	float bind;
	if (power > 0.0) bind = sqrt(dot(m, m));
	else {if (prop < 1.0) bind = m.x; else bind = m.y;}

	vec2 uv;
	if (power > 0.0)
		uv = m + normalize(d) * tan(r * power) * bind / tan( bind * power);
	else if (power < 0.0)
		uv = m + normalize(d) * atan(r * -power * 10.0) * bind / atan(-power * bind * 10.0);
	else uv = p;

	// chromatic abberation
	float r1 = texture(scene, vec2(uv.x + (0.002 * r * CHROMATIC_ABBERATION_STRENGTH), uv.y * prop)).r;
	float g1 = texture(scene, vec2(uv.x + (0.003 * r * CHROMATIC_ABBERATION_STRENGTH), uv.y * prop)).g;
	float b1 = texture(scene, vec2(uv.x + (0.005 * r * CHROMATIC_ABBERATION_STRENGTH), uv.y * prop)).b;
	return vec4(r1, g1, b1, 1.0);
}

// adapted from https://www.shadertoy.com/view/lslGzl
vec4 tonemap(vec4 color)
{
	float gamma = 2.2;
	float white = 1.2;
	float luma = dot(color, vec4(0.2126, 0.7152, 0.0722, 1.0));
	float toneMappedLuma = luma * (1. + luma / (white*white)) / (1. + luma);
	color *= toneMappedLuma / luma;
	color = pow(color, vec4(1. / gamma, 1. / gamma, 1. / gamma, 1.0));
	color = color - exp(-color * 8.5) * 1.25;
	return color;
}

vec4 colorGrading(vec4 color)
{
	float l = luma(color);
	return saturate(
		mix(
			color,
			color +
			vec4(
				SCurve(color.r, 1, 2.25) + 0.025 + 0.5 * l,
				SCurve(color.g, 0.5, 2) + 0.035 + 0.35 * l, 
				SCurve(color.b, 1, 1.5) + 0.03 + 0.5 * l, 
				0.0
			),
			COLOR_GRADE_STRENGTH
		),
		clamp(l * BRIGHTNESS_SATURATION_STRENGTH, 0, MAX_SATURATION)
	);
}

vec4 vignette(vec4 color)
{
	float dist = sqrt(distance(texCoord, vec2(0.5)));
	if (dist < VIGNETTE_DISTANCE)
	{
		return color;
	}
	return color - (dist - VIGNETTE_DISTANCE) * VIGNETTE_STRENGTH;
}

void main(void)
{
	outColor = fishEye(outColor, texCoord);
	outColor = colorGrading(outColor);
	outColor = tonemap(outColor);
	outColor = vignette(outColor);
}

